@@ -6,6 +6,7 @@ use crate::bitmap::store::array_store::visitor::{CardinalityCounter, VecWriter};
 use core::cmp::Ordering;
 use core::cmp::Ordering::*;
 use core::fmt::{Display, Formatter};
+use core::mem::size_of;
 use core::ops::{BitAnd, BitAndAssign, BitOr, BitXor, RangeInclusive, Sub, SubAssign};
 
 #[cfg(not(feature = "std"))]
@@ -15,9 +16,12 @@ use alloc::vec::Vec;
 use alloc::boxed::Box;
 
 use super::bitmap_store::{bit, key, BitmapStore, BITMAP_LENGTH};
+use super::Interval;
+
+pub(crate) const ARRAY_ELEMENT_BYTES: usize = 2;
 
 #[derive(Clone, Eq, PartialEq)]
-pub struct ArrayStore {
+pub(crate) struct ArrayStore {
     vec: Vec<u16>,
 }
 
@@ -26,6 +30,15 @@ impl ArrayStore {
         ArrayStore { vec: vec![] }
     }
 
+    pub fn serialized_byte_size(cardinality: u64) -> usize {
+        cardinality as usize * ARRAY_ELEMENT_BYTES
+    }
+
+    pub fn byte_size(&self) -> usize {
+        Self::serialized_byte_size(self.len())
+    }
+
+    #[cfg(feature = "std")]
     pub fn with_capacity(capacity: usize) -> ArrayStore {
         ArrayStore { vec: Vec::with_capacity(capacity) }
     }
@@ -52,6 +65,33 @@ impl ArrayStore {
         }
     }
 
+    pub fn from_lsb0_bytes(bytes: &[u8], byte_offset: usize, bits_set: u64) -> Self {
+        type Word = u64;
+
+        let mut vec = Vec::with_capacity(bits_set as usize);
+
+        let chunks = bytes.chunks_exact(size_of::<Word>());
+        let remainder = chunks.remainder();
+        for (index, chunk) in chunks.enumerate() {
+            let bit_index = (byte_offset + index * size_of::<Word>()) * 8;
+            let mut word = Word::from_le_bytes(chunk.try_into().unwrap());
+
+            while word != 0 {
+                vec.push((word.trailing_zeros() + bit_index as u32) as u16);
+                word &= word - 1;
+            }
+        }
+        for (index, mut byte) in remainder.iter().copied().enumerate() {
+            let bit_index = (byte_offset + (bytes.len() - remainder.len()) + index) * 8;
+            while byte != 0 {
+                vec.push((byte.trailing_zeros() + bit_index as u32) as u16);
+                byte &= byte - 1;
+            }
+        }
+
+        Self::from_vec_unchecked(vec)
+    }
+
     #[inline]
     pub fn insert(&mut self, index: u16) -> bool {
         self.vec.binary_search(&index).map_err(|loc| self.vec.insert(loc, index)).is_err()
@@ -192,6 +232,15 @@ impl ArrayStore {
         visitor.into_inner()
     }
 
+    pub fn intersection_len_interval(&self, interval: &Interval) -> u64 {
+        if interval.is_full() {
+            return self.len();
+        }
+        let start_id = self.vec.partition_point(|&f| f < interval.start());
+        let end_id = self.vec.partition_point(|&f| f <= interval.end());
+        (end_id.saturating_sub(start_id)) as u64
+    }
+
     pub fn to_bitmap_store(&self) -> BitmapStore {
         let mut bits = Box::new([0; BITMAP_LENGTH]);
         let len = self.len();
@@ -230,7 +279,7 @@ impl ArrayStore {
         self.vec.get(n as usize).cloned()
     }
 
-    pub fn iter(&self) -> core::slice::Iter<u16> {
+    pub fn iter(&'_ self) -> core::slice::Iter<'_, u16> {
         self.vec.iter()
     }
 
@@ -258,6 +307,23 @@ impl ArrayStore {
         }
         self.vec.truncate(pos);
     }
+
+    pub(crate) fn internal_validate(&self) -> Result<(), &'static str> {
+        if self.vec.is_empty() {
+            return Err("zero cardinality in array container");
+        }
+        if self.vec.len() > super::ARRAY_LIMIT as usize {
+            return Err("array cardinality exceeds ARRAY_LIMIT");
+        }
+        for window in self.vec.windows(2) {
+            let &[first, second] = window else { unreachable!() };
+            if first >= second {
+                return Err("array elements not strictly increasing");
+            }
+        }
+
+        Ok(())
+    }
 }
 
 impl Default for ArrayStore {
@@ -356,7 +422,7 @@ impl BitAndAssign<&Self> for ArrayStore {
             let mut i = 0;
             self.retain(|x| {
                 i += rhs.iter().skip(i).position(|y| *y >= x).unwrap_or(rhs.vec.len());
-                rhs.vec.get(i).map_or(false, |y| x == *y)
+                rhs.vec.get(i).is_some_and(|y| x == *y)
             });
         }
     }
@@ -431,6 +497,7 @@ mod tests {
         match s {
             Store::Array(vec) => vec.vec,
             Store::Bitmap(bits) => bits.to_array_store().vec,
+            Store::Run(runs) => runs.iter().collect(),
         }
     }
 
@@ -438,6 +505,7 @@ mod tests {
         match s {
             Store::Array(vec) => Store::Bitmap(vec.to_bitmap_store()),
             Store::Bitmap(..) => s,
+            Store::Run(runs) => Store::Bitmap(runs.to_bitmap()),
         }
     }
 
